@@ -6,12 +6,6 @@ use APR::Table;
 use Apache2::RequestRec;
 use Apache2::Response;
 
-my %Handler = (
-    'Content-length' => 'set_content_length',
-    'Content-Type'   => 'content_type',
-    'Status'         => 'status_line',
-);
-
 sub request_rec {
     $_[0]->query->r;
 }
@@ -21,7 +15,7 @@ sub finalize {
     my $headers     = $self->as_arrayref;
     my $request_rec = $self->request_rec;
 
-    my $status = $self->status || '200';
+    my $status = {@$headers}->{'Status'} || '200';
        $status =~ s/\D*$//;
 
     my $headers_out = $status >= 200 && $status < 300 ? 'headers_out' : 'err_headers_out';  
@@ -33,8 +27,14 @@ sub finalize {
         my $field = $headers->[$i];
         my $value = $self->process_newline( $headers->[$i+1] );
 
-        if ( my $handler = $Handler{$field} ) {
-            $request_rec->$handler( $value );
+        if ( $field eq 'Content-Type' ) {
+            $request_rec->content_type( $value );
+        }
+        elsif ( $field eq 'Content-length' ) {
+            $request_rec->set_content_length( $value );
+        }
+        elsif ( $field eq 'Status' ) {
+            $request_rec->status_line( $value );
         }
         else {
             $headers_out->add( $field => $value );
